// Farmbauer/ Kneil 2007/2008

#include "Hilfs.h"


//     -----     d e l a y      -----

void delay_s(unsigned char n)
{
    do {
	delay_ms(250);
	delay_ms(250);
	delay_ms(250);
	delay_ms(250);
    } while(--n);
}

void delay_ms(unsigned char t)
{
    do {
	_delay_ms(1);
    } while (t--);
}

void delay_us(unsigned char t)
{
    do {
	_delay_us(1);
    } while (t--);
}

/*
 * JTAG-Interface eines ATmega32 per Software deaktivieren
 * und damit die JTAG-Pins an PORTC fr "general I/O" nutzbar machen
 * ohne die JTAG-Fuse-Bit zu aendern. Dazu ist eine "timed sequence"
 * einzuhalten:
 * Das JTD-Bit muss zweimal innerhalb von 4 Taktzyklen geschrieben
 * werden. Ein Interrupt zwischen den beiden Schreibzugriffen wuerde
 * die erforderliche Sequenz "brechen", das JTAG-Interface bliebe
 * weiterhin aktiv und die IO-Pins weiterhin fr JTAG reserviert.
 */
void DisableJTAG(void)
 {
 uint8_t tmp_sreg;  // temporaerer Speicher fuer das Statusregister
 tmp_sreg = SREG;   // Statusregister sichern
 cli();             // Interrupts global deaktivieren
    asm volatile(
  "push r24\n\t"
  "lds    r24, 0x0054\n\t"
  "ori    r24, 0x80\n\t"
     "sts    0x0054, r24\n\t"
     "sts    0x0054, r24\n\t" // 2 mal in Folge, vgl. Datenblatt
  "pop r24\n\t"
     ::);
 SREG = tmp_sreg;// Status-Register wieder herstellen, I-Flag rcksetzen
 }



//     -----     i n i t     -----

void initAll(){
    unsigned char i,j, pattern=0;
    unsigned short adres=0;


    // disable JTAG, we need the pin
    DisableJTAG();

	// configure port a
    PORTA = 0x1e; // enable switch pullups PA1..PA4
    DDRA = 0xe0; // RS,E,R/W out, PA1..PA4 in (switches), PA0 in (ADC)

	// configure port b
    PORTB = 0x00;
    DDRB = 0xff; // all bits out (Leds 0..7)

	// LCD display init
    LCD_init8bit();
    LCD_cmd8bit(0x0c); // Display on, Cursor off, Blink off

	// ADC initialisieren
	ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0); // Frequenzvorteiler (clk/8) 
    // setzen auf 8 (1) und ADC aktivieren (1)
    ADMUX |= (1<<REFS1) | (1<<REFS0) | 0; // interne Referenzspannung nutzen

    ADCSRA |= (1<<ADSC);             // eine ADC-Wandlung 
    while ( ADCSRA & (1<<ADSC) ) ;   // auf Abschluss der Konvertierung warten 
    adres = ADCW; // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nächsten Wandlung
                  // nicht übernommen.


	// Willkommensgru ausgeben
    LCD_write(" Fachhochschule");
    LCD_pos(0x40);
    LCD_write("   Regensburg");

    // leds blinken
    for(i=0;i<8;i++) {
      for(j=0;j<8-i;j++) {
        PORTB = pattern | 1<<j;
        delay_ms(120);
      }
      pattern |= 1<<(7-i);
    }
    delay_s(1);
    for(i=0;i<8;i++) {
      pattern &= ~(1<<(7-i));
      for(j=7-i;j<8;j++) {
        PORTB = pattern | 1<<j;
        delay_ms(120);
      }
    }


	
  return;
}

//     -----     l c d   d i s p l a y     -----

#define RS PA7
#define E  PA6
#define RW PA5

void LCD_cmd8bit(unsigned char c)
{
    PORTA &= ~((1<<RS)|(1<<RW));
    PORTC = c;
    delay_us(1);
    PORTA |= (1<<E);
    delay_us(1);
    PORTA &= ~(1<<E);
    delay_us(50);
    if(c==0x01 || (c&0xfe)==0x02)
	delay_ms(2);
}

void LCD_data8bit(unsigned char c)
{
    PORTA &= ~(1<<RW);
    PORTA |= (1<<RS);
    PORTC = c;
    delay_us(1);
    PORTA |= (1<<E);
    delay_us(1);
    PORTA &= ~(1<<E);
    delay_us(50);
}

void LCD_init8bit(void)
{
    // setup IO Pins
    PORTC = 0x00;
    DDRC = 0xff;
    PORTA &= ~((1<<RS)|(1<<E)|(1<<RW));
    DDRA = (1<<RS)|(1<<E)|(1<<RW);

    delay_ms(40);
    // reset
    LCD_cmd8bit(0x30); // 8-bit
    LCD_cmd8bit(0x30); // 8-bit
    LCD_cmd8bit(0x30); // 8-bit
    // setup
    LCD_cmd8bit(0x38); // 8-bit, 2 lines, 5x8
    LCD_cmd8bit(0x0e); // cursor on, display on, blink off
    LCD_cmd8bit(0x02); // home
    LCD_cmd8bit(0x01); // clear
    //LCD_cmd8bit(0x06); // cursor direction

}

void LCD_write(char *text)
{
    while(*text) {
	LCD_data8bit(*text);
	text++;
    }
}

void lcdPrint(int line, int col, char *text){
	if(line <=0){
		LCD_pos(0x0+col%17);
	}else{
		LCD_pos(0x40+col%17);
	}
	LCD_write(text);
}

unsigned short getAdc(){
	unsigned short adres;
     // start single conversion and wait for it
	 ADCSRA |= (1<<ADSC);
     while ( ADCSRA & (1<<ADSC) ) delay_us(1);

     adres = ADCW & 0x3ff; // 10 Bit; values 0..1023
	 return adres;
}
  
